Package: pfwim
Title: Food Web Construction for Paleo Communities
Version: 1.0.2
Authors@R: c(
    person("Tanya", "Strydom", role = c("aut", "cre"), email = "t.strydom@sheffield.ac.uk",
           comment = c(ORCID = "0000-0001-6067-1349")),
    person("Jack O.", "Shaw", role = c("ctb"),
           comment = c(ORCID = "0000-0002-2725-2334")),
    person("Alexander M.", "Dunhill", role = "ctb",
           comment = c(ORCID = "0000-0002-8680-9163")),
    person("Andrew", "Beckerman", role = "ctb",
           comment = c(ORCID = "0000-0002-7859-8394")),
    person("Baran", "Karapunar", role = "ctb",
           comment = c(ORCID = "0000-0001-9711-1492")),
    person("Jennifer A.", "Dunne", role = "ctb",
           comment = c(ORCID = "0000-0002-3789-9561")),
    person("Pincelli M.", "Hull", role = "ctb",
           comment = c(ORCID = "0000-0001-8607-4817"))
  )
Description: R tools for reconstructing paleo food webs from species traits and size rules.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
LazyData: true
Imports: dplyr, stats, tibble, tidyr, tidyselect
Suggests: igraph, knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://beckslab.github.io/pfwim/,
        https://github.com/BecksLab/pfwim
NeedsCompilation: no
Packaged: 2026-04-25 10:15:13 UTC; tanyastrydom
Author: Tanya Strydom [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6067-1349>),
  Jack O. Shaw [ctb] (ORCID: <https://orcid.org/0000-0002-2725-2334>),
  Alexander M. Dunhill [ctb] (ORCID:
    <https://orcid.org/0000-0002-8680-9163>),
  Andrew Beckerman [ctb] (ORCID: <https://orcid.org/0000-0002-7859-8394>),
  Baran Karapunar [ctb] (ORCID: <https://orcid.org/0000-0001-9711-1492>),
  Jennifer A. Dunne [ctb] (ORCID:
    <https://orcid.org/0000-0002-3789-9561>),
  Pincelli M. Hull [ctb] (ORCID: <https://orcid.org/0000-0001-8607-4817>)
Maintainer: Tanya Strydom <t.strydom@sheffield.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-28 19:30:24 UTC
Built: R 4.5.2; ; 2026-04-28 20:13:25 UTC; unix

# Generated by roxygen2: do not edit by hand

export(infer_edgelist)
export(powerlaw_prey)
importFrom(dplyr,any_vars)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter_all)
importFrom(dplyr,sample_n)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(tidyr,"%>%")
importFrom(tidyselect,all_of)

# PFWIM: Paleo Food Web Inference Model <img src="man/figures/logo.png" align="right" width="120" />

`PFWIM` is an R package for inferring predator–prey interactions using trait-based 
rules and generating hypothetical realised food webs. It implements 
the **Paleo Food-web Inference Model (Shaw 2024)** and includes tools for categorical 
trait matching, numerical predator–prey size rules, and power-law downsampling of 
realised webs.

## Installation

```
# Install from CRAN (when available)
install.packages("pfwim")

# Or install development version from GitHub
# install.packages("devtools")
devtools::install_github("BecksLab/pfwim")
```

## Load Example Data

```
library(pfwim)

# Trait data
data("traits", package = "pfwim")

# Trait combination rules
data("feeding_rules", package = "pfwim")

head(traits)
head(feeding_rules)
```

## Inferring a consumer–resource edgelist

Use `infer_edgelist()` to infer feasible interactions:

```
edgelist <- infer_edgelist(
  data = traits,
  cat_combo_list = feeding_rules,
  col_taxon = "species",
  certainty_req = "all",
)

# Show first few interactions
head(edgelist)
```

## Generate hypothetical realised webs

Create a series of hypothetical webs using `powerlaw_prey()`

```
webs <- powerlaw_prey(
  el = edgelist,
  n_samp = 5,
  y = 2.5
)

# Inspect first web
webs[[1]]
```

Each element of `webs` is a matrix representing a realised food web generated 
from the inferred interactions

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(dplyr)
library(tidyr)
library(pfwim)

# Load example data included in the package
data("traits", package = "pfwim")
data("feeding_rules", package = "pfwim")

## ----trait-preview------------------------------------------------------------
# Preview the input structure
head(traits)

## ----rules-preview------------------------------------------------------------
# Preview the input structure
head(feeding_rules)

## ----infer-edgelist-----------------------------------------------------------

edgelist <- infer_edgelist(
  data = traits,
  cat_combo_list = feeding_rules,
  col_taxon = "species",
  certainty_req = "all",
  hide_printout = TRUE
)

head(edgelist)

## ----powerlaw-webs------------------------------------------------------------
realised_webs <- powerlaw_prey(
  el = edgelist,
  n_samp = 5,
  y = 2.5
)

# Inspect first realised web
realised_webs[[1]]

## ----custom-func--------------------------------------------------------------
# Define a custom in-degree distribution function
custom_func <- function(r, M, y) (M - r + 1)^(-y)

realised_webs_custom <- powerlaw_prey(
  el = edgelist,
  n_samp = 5,
  y = 2,
  func = custom_func
)

# First custom web
realised_webs_custom[[1]]

## ----setup, message=FALSE, warning=FALSE--------------------------------------

library(pfwim)
library(dplyr)
library(igraph)

data("traits", package = "pfwim")
data("feeding_rules", package = "pfwim")


## -----------------------------------------------------------------------------


metaweb_el <- infer_edgelist(
  data = traits,
  cat_combo_list = feeding_rules,
  col_taxon = "species",
  certainty_req = "all",
  hide_printout = TRUE
)

head(metaweb_el)


## -----------------------------------------------------------------------------


metaweb_graph <- graph_from_data_frame(
  metaweb_el,
  directed = TRUE
)

metaweb_graph


## -----------------------------------------------------------------------------


set.seed(66)

plot(
  metaweb_graph,
  vertex.size = 35,
  vertex.label.cex = 0.6,
  edge.arrow.size = 0.3,
  layout = layout_with_fr(metaweb_graph),
  main = "Metaweb"
)

## ----modify_traits------------------------------------------------------------

library(pfwim)
library(dplyr)

# Adding body mass to our traits
traits_numeric <- traits %>%
  mutate(body_mass = case_when(
    species == "polar_bear" ~ 450,
    species == "seal"       ~ 100,
    species == "orca"       ~ 3000,
    species == "cod"        ~ 5,
    species == "plankton"   ~ 0.0001,
    TRUE                    ~ 10 # Default for others
  ))

head(traits_numeric)


## ----size_rule----------------------------------------------------------------

my_size_rule <- function(res_size, con_size) {
  ratio <- con_size / res_size
  ifelse(ratio >= 2 & ratio <= 100, 1, 0)
}


## ----build_list---------------------------------------------------------------

edgelist_size <- infer_edgelist(
  data = traits_numeric,
  cat_combo_list = feeding_rules,
  col_taxon = "species",
  col_num_size = "body_mass",    # Point to the numeric column
  num_size_rule = my_size_rule,   # Apply our custom logic
  certainty_req = "all",
  hide_printout = TRUE
)

head(edgelist_size)

#' Sample integers according to a discrete probability distribution
#'
#' Generates random draws from 1:M with probabilities defined by a
#' user-specified function (default is the PFWIM mixed exponential–power law
#' distribution).
#'
#' @param M Integer. Maximum value to sample (upper bound of 1:M).
#' @param y Numeric. Parameter controlling the distribution shape.
#' @param func Function. Probability function of r (1:M), M, and y.
#'   Default is the PFWIM exponential–power law function:
#'   \code{function(r, M, y) exp(-r / (exp((y - 1) * (log(M) / y))))}.
#' @param n_samp Integer. Number of random draws.
#'
#' @return Integer vector of length \code{n_samp} with values in 1:M.
#' @keywords internal
#'
#' @examples
#' # Generate 3 replicate webs from a small edgelist
#' edgelist <- data.frame(
#'   res_node_node_name_inferred = c("plankton","plant_1","plant_2"),
#'   con_node_node_name_inferred = c("cod","rat","deer")
#' )
#' webs <- powerlaw_prey(edgelist, n_samp = 3, y = 2.5)
sample_pdf <- function(M = 100,
                       y = 2.5,
                       func = function(r, M, y) exp(-r / (exp((y - 1) * (log(M) / y)))),
                       n_samp = 100) {

  probs <- sapply(1:M, func, M = M, y = y)
  probs <- probs / sum(probs)

  sample(1:M, n_samp, replace = TRUE, prob = probs)
}

#' Generate hypothetical realised webs using a power-law link distribution
#'
#' PFWIM generates a series of replicate hypothetical realised food webs by
#' reducing the feasible links for each consumer to match a target link
#' distribution. The default distribution is a mixed exponential–power law
#' in-degree distribution as described in Shaw (2024) and Roopnarine (2006).
#'
#' @param el Data frame or matrix containing a feasible consumer–resource
#'   edgelist. Column 1 = resource, Column 2 = consumer.
#' @param n_samp Integer. Number of replicate realised webs to generate. Default = 50.
#' @param y Numeric. Parameter controlling the shape of the power-law distribution.
#'   Default = 2.5.
#' @param func Function. Probability function of the in-degree `r`, total prey
#'   richness `M`, and parameter `y`. Must return a numeric value > 0.
#'   Default:
#'   \code{function(r, M, y) exp(-r / (exp((y - 1) * (log(M) / y))))}.
#'
#' @return A list of length \code{n_samp}. Each element is a data frame
#'   representing a realised food web edgelist with two columns:
#'   \describe{
#'     \item{resource}{Resource species}
#'     \item{consumer}{Consumer species}
#'   }
#'
#' @details For each consumer in `el`, the number of prey links in a realised
#' web is sampled according to the distribution defined by `func`. The sampled
#' prey are drawn randomly without exceeding the maximum feasible prey for that
#' consumer.
#'
#' @references
#' Shaw, J. (2024). PFWIM: Paleo Food-web Inference Model. *Preprint*.
#' Roopnarine, P. (2006). *Palaeoecology and food-web structure in fossil communities*.
#'
#' @examples
#' # Infer a minimal edgelist
#' edgelist <- infer_edgelist(
#'   data = data.frame(
#'     species = c("plankton","plant_1","plant_2","cod","rat","deer"),
#'     feeding = c("primary","primary","primary","secondary","secondary","secondary")
#'   ),
#'   cat_combo_list = data.frame(
#'     trait_type_resource = c("feeding","feeding","feeding"),
#'     trait_resource = c("primary","primary","primary"),
#'     trait_type_consumer = c("feeding","feeding","feeding"),
#'     trait_consumer = c("secondary","secondary","secondary")
#'   ),
#'   col_taxon = "species",
#'   certainty_req = "all"
#' )
#'
#' # Generate realized webs
#' webs <- powerlaw_prey(edgelist, n_samp = 3, y = 2.5)
#'
#' @export
powerlaw_prey <- function(el,
                          n_samp = 50,
                          y = 2.5,
                          func = function(r, M, y) exp(-r / (exp((y - 1) * (log(M) / y))))) {

  con_node_node_name_inferred <- NULL

  # Ensure input is a data frame
  edgelist <- as.data.frame(el)
  colnames(edgelist) <- c("resource", "consumer")

  consumer_list <- split(edgelist, edgelist$consumer)

  # Initialise list to store realised webs
  web_list <- vector("list", n_samp)

  for (consumer in names(consumer_list)) {

    min <- consumer_list[[consumer]]

    M_consumer <- nrow(min)

    sampled_degree <- sample_pdf(
      M = M_consumer,
      y = y,
      func = func,
      n_samp = n_samp
    )

    for (j in seq_len(n_samp)) {

      n_links <- min(M_consumer, sampled_degree[j])

      sampled_rows <- min[sample(M_consumer, n_links), ]

      if (is.null(web_list[[j]])) {
        web_list[[j]] <- sampled_rows
      } else {
        web_list[[j]] <- rbind(web_list[[j]], sampled_rows)
      }

    }
  }

  # give named outputs
  names(web_list) <- paste0("web_", seq_len(n_samp))

  return(web_list)
}

test_that("powerlaw_prey runs", {

  expect_no_error(
    powerlaw_prey(edgelist, n_samp = 10)
  )

})

test_that("powerlaw_prey returns list of webs", {

  webs <- powerlaw_prey(edgelist, n_samp = 10)

  expect_true(is.list(webs))
  expect_equal(length(webs), 10)

})

test_that("each replicate web is a data.frame", {

  webs <- powerlaw_prey(edgelist, n_samp = 5)

  expect_true(all(sapply(webs, is.data.frame)))

})

test_that("prey counts do not exceed feasible prey", {

  webs <- powerlaw_prey(edgelist, n_samp = 5)

  max_possible <- table(edgelist$taxon_consumer)

  for (web in webs) {

    web <- as.data.frame(web)
    colnames(web) <- c("resource","consumer")

    realised <- table(factor(web$consumer, levels = names(max_possible)))

    expect_true(all(realised <= max_possible))
  }

})

#' Infer trophic interactions using PFWIM trait rules
#'
#' Infers a consumer–resource edgelist using categorical trait matching rules
#' based on the PFWIM (Paleo Food Web Inference Model) described in
#' Shaw (2024). Interactions are inferred by comparing resource and consumer
#' trait combinations against a set of allowed trait rules.
#'
#' Trait matching is performed across all trait types present in
#' `cat_combo_list`. For each potential taxon pair, the number of satisfied
#' trait rules is calculated. An interaction is inferred when the number of
#' satisfied rules meets the required threshold defined by `certainty_req`.
#'
#' Optionally, a numerical predator–prey size rule can also be applied using
#' `num_size_rule`.
#'
#' @param data A `data.frame` containing taxa and associated trait values.
#' Each row represents a taxon and each column represents a trait.
#'
#' @param cat_combo_list A `data.frame` defining allowed consumer–resource
#' trait combinations. Must contain columns:
#' \describe{
#' \item{trait_type_resource}{Resource trait category}
#' \item{trait_resource}{Resource trait value}
#' \item{trait_type_consumer}{Consumer trait category}
#' \item{trait_consumer}{Consumer trait value}
#' }
#'
#' @param col_taxon Character string indicating the column containing taxon
#' names in `data`. Default `"taxon"`.
#'
#' @param col_num_size Optional column name containing numerical size values
#' for taxa. Used when applying a numeric predator–prey size rule.
#'
#' @param cat_trait_types Optional character vector specifying a subset of
#' categorical trait columns to use. If `NULL`, all traits appearing in
#' `cat_combo_list` are used.
#'
#' @param num_size_rule Optional function defining the predator–prey size
#' feasibility rule. The function must accept two numeric vectors:
#' `(resource_size, consumer_size)` and return `1` for feasible interactions
#' and `0` otherwise.
#'
#' Example:
#'
#' `function(res_size, con_size) { ifelse(res_size <= con_size, 1, 0) }`
#'
#' @param certainty_req Defines how many trait rules must be satisfied for an
#' interaction to be considered feasible.
#' \describe{
#' \item{"all"}{All trait types must match}
#' \item{numeric}{Minimum number of matching trait types required}
#' }
#'
#' @param allow_self Logical. If TRUE, allows interactions where the resource
#' and consumer are the same taxon (self-loops). Default is FALSE.
#'
#' @param return_full_matrix Logical. If `TRUE`, returns the full matrix of
#' taxon pairs and the number of matching trait rules. If `FALSE`, returns
#' only inferred interactions as an edgelist.
#'
#' @param print_dropped_taxa Logical. If `TRUE`, prints taxa that were removed
#' from the inferred food web because they have no feasible interactions.
#'
#' @param hide_printout Logical. If `TRUE`, suppresses progress messages.
#'
#' @param ... Additional arguments reserved for future extensions.
#'
#' @return
#' If `return_full_matrix = FALSE`:
#'
#' A two-column matrix containing:
#' \describe{
#' \item{taxon_resource}{Resource taxon}
#' \item{taxon_consumer}{Consumer taxon}
#' }
#'
#' If `return_full_matrix = TRUE`:
#'
#' A matrix containing all taxon pairs and the number of trait rules satisfied.
#'
#' @details
#' The function evaluates all possible consumer–resource taxon pairs and
#' determines interaction feasibility using categorical trait rules and,
#' optionally, a numerical size constraint. The final edgelist includes only
#' interactions meeting the certainty requirement.
#'
#' @references
#' Shaw, J. (2024). PFWIM: Paleo Food web Inference Model. *Preprint*.
#'
#' @examples
#' infer_edgelist(
#'   data = traits,
#'   cat_combo_list = feeding_rules,
#'   col_taxon = "species",
#'   certainty_req = "all"
#' )
#'
#' @export
infer_edgelist <- function(data,
                           cat_combo_list,
                           col_taxon = "taxon",
                           col_num_size = NULL,
                           cat_trait_types = NULL,
                           num_size_rule = NULL,
                           certainty_req = "all",
                           allow_self = TRUE,
                           return_full_matrix = FALSE,
                           print_dropped_taxa = FALSE,
                           hide_printout = FALSE,
                           ...) {

  # Hide "no visible binding for global variable" comment
  . <- Var1 <- Var2 <- pres_sum <- size_consumer <- size_resource <- taxon_consumer <- taxon_resource <- trait <- trait_type <- trait_type_consumer <- trait_type_interaction <- trait_type_pres <- trait_value_pres <- trait_type_resource <- trait_consumer <- NULL

  # Check inputs

  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }

  if (!is.data.frame(cat_combo_list)) {
    stop("`cat_combo_list` must be a data.frame.", call. = FALSE)
  }

  required_cols <- c(
    "trait_type_resource",
    "trait_resource",
    "trait_type_consumer",
    "trait_consumer"
  )

  missing_cols <- setdiff(required_cols, colnames(cat_combo_list))
  if (length(missing_cols) > 0) {
    stop(
      paste0(
        "`cat_combo_list` is missing required columns: ",
        paste(missing_cols, collapse = ", ")
      ),
      call. = FALSE
    )
  }

  if (!col_taxon %in% colnames(data)) {
    stop(
      paste0("Column `", col_taxon, "` not found in `data`."),
      call. = FALSE
    )
  }

  if (!is.null(col_num_size)) {
    if (!col_num_size %in% colnames(data)) {
      stop(
        paste0("Column `", col_num_size, "` not found in `data`."),
        call. = FALSE
      )
    }

    if (is.null(num_size_rule)) {
      stop(
        "`num_size_rule` must be provided when `col_num_size` is specified.",
        call. = FALSE
      )
    }

    if (!is.function(num_size_rule)) {
      stop("`num_size_rule` must be a function.", call. = FALSE)
    }
  }

  if (!is.null(cat_trait_types)) {
    if (!all(cat_trait_types %in% colnames(data))) {
      stop(
        "All `cat_trait_types` must be columns present in `data`.",
        call. = FALSE
      )
    }
  }

  if (!(certainty_req == "all" || is.numeric(certainty_req))) {
    stop("`certainty_req` must be 'all' or a numeric value.", call. = FALSE)
  }

  if (is.numeric(certainty_req) && certainty_req <= 0) {
    stop("`certainty_req` must be greater than 0.", call. = FALSE)
  }

  names(data)[names(data) == col_taxon] <- "taxon"
  col_taxon <- "taxon"

  trait_cats <- tolower(unique(c(cat_combo_list[, c("trait_type_resource")], cat_combo_list[, c("trait_type_consumer")])))
  trait_cats_cons <- tolower(unique(c(cat_combo_list[, c("trait_type_consumer")])))
  col_taxon <- tolower(col_taxon)

  if (is.null(cat_trait_types)) {
  } else {
    trait_cats <- cat_trait_types
    trait_cats_cons <- cat_trait_types
  }

  fd <- as.data.frame(data)
  colnames(fd) <- tolower(colnames(fd))


  # Run for all trait types
  fw_match_traits <- c()
  for (i in trait_cats_cons) {
    col_combo_list_mini <- cat_combo_list %>% dplyr::filter(trait_type_consumer == i)

    res_cols <- unique(col_combo_list_mini$trait_type_resource)

    res_taxa <- fd %>%
      dplyr::select(dplyr::all_of(c(col_taxon, res_cols)))
    con_taxa <- fd %>%
      dplyr::select(dplyr::all_of(c(col_taxon, i))) %>%
      filter_all(any_vars(!is.na(.)))
    con_taxa <- con_taxa[Reduce(`&`, lapply(con_taxa, function(x) !(is.na(x) | x == ""))), ]

    res_taxa_longer <- res_taxa %>%
      dplyr::mutate_all(., as.character) %>%
      tidyr::pivot_longer(-dplyr::all_of(col_taxon), names_to = "trait_type", values_to = "trait") %>%
      dplyr::filter(trait_type %in% col_combo_list_mini[, c("trait_type_resource")])

    con_taxa_longer <- con_taxa %>%
      dplyr::mutate_all(., as.character) %>%
      tidyr::pivot_longer(-dplyr::all_of(col_taxon), names_to = "trait_type", values_to = "trait") %>%
      dplyr::filter(trait != "primary")

    # create crossing table
    crossing_taxa <- tidyr::crossing(
      res_taxa_longer %>% dplyr::rename_with(~paste0(.x, "_resource")),
      con_taxa_longer %>% dplyr::rename_with(~paste0(.x, "_consumer"))
    )

    # first join (trait type feasibility)
    crossing_taxa <- dplyr::left_join(
      crossing_taxa,
      col_combo_list_mini %>%
        dplyr::select(trait_type_resource, trait_type_consumer, trait_consumer) %>%
        dplyr::mutate(trait_type_pres = 1),
      by = c(
        "trait_type_resource",
        "trait_type_consumer",
        "trait_consumer"
      ),
      relationship = "many-to-many"
    )

    # filter valid types
    crossing_taxa <- crossing_taxa %>%
      dplyr::filter(trait_type_pres == 1) %>%
      dplyr::distinct()

    # second join (trait value feasibility)
    crossing_taxa <- dplyr::left_join(
      crossing_taxa,
      col_combo_list_mini %>%
        dplyr::mutate(trait_value_pres = 1),
      by = c(
        "trait_type_resource",
        "trait_type_consumer",
        "trait_consumer",
        "trait_resource"
      ),
      relationship = "many-to-many"
    )

    # final filtering
    crossing_taxa <- crossing_taxa %>%
      dplyr::filter(trait_value_pres == 1) %>%
      dplyr::mutate(trait_type_interaction = i) %>%
      dplyr::select(
        taxon_resource,
        taxon_consumer,
        trait_type_interaction,
        trait_value_pres
      )


    fw_match_traits <- rbind(fw_match_traits, crossing_taxa)
    if (!hide_printout) print(i)
  }

  # If feasibility of interaction also defined by a numerical size rule, then utilize
  if (is.null(col_num_size)) {

  } else {
    fw_size_pastes <- unique(paste(fd[, c(col_taxon)], fd[, c(col_num_size)], sep = "SEP"))
    fw_size_expand <- as.data.frame(expand.grid(fw_size_pastes, fw_size_pastes)) %>%
      tidyr::separate(Var1, c("taxon_resource", "size_resource"), sep = "SEP", convert = TRUE) %>%
      tidyr::separate(Var2, c("taxon_consumer", "size_consumer"), sep = "SEP", convert = TRUE) %>%
      dplyr::mutate(trait_value_pres = num_size_rule(size_resource, size_consumer), trait_type_interaction = col_num_size) %>%
      dplyr::filter(trait_value_pres == 1) %>%
      dplyr::select(taxon_resource, taxon_consumer, trait_value_pres, trait_type_interaction)
    fw_match_traits <- bind_rows(fw_match_traits, fw_size_expand)
    if (!hide_printout) print(col_num_size)
  }


  fw_match_traits2 <- fw_match_traits %>%
    dplyr::distinct() %>%
    tidyr::pivot_wider(names_from = trait_type_interaction, values_from = trait_value_pres)
  fw_match_traits2$pres_sum <- fw_match_traits2 %>%
    dplyr::select(all_of(unique(fw_match_traits$trait_type_interaction))) %>%
    rowSums(na.rm = TRUE)

  # How many trait combinations need to be matched in order for an interaction to be viable
  if (certainty_req == "all") {
    certainty_val <- length(unique(fw_match_traits$trait_type_interaction))
  } else {
    certainty_val <- certainty_req
    certainty_val <- ifelse(certainty_val > length(unique(fw_match_traits$trait_type_interaction)), length(unique(fw_match_traits$trait_type_interaction)), certainty_val)
  }

  # remove cannibalism if allow_self is set to false
  if (!allow_self) {
    fw_match_traits2 <- fw_match_traits2 %>%
      dplyr::filter(taxon_resource != taxon_consumer)
  }
  # --- FIX ENDS HERE ---

  # Return full matrix with trait feasibility indicated or not
  if (return_full_matrix == TRUE) {
    # Simply assign the filtered/processed traits to the output variable
    fw_match_traits3 <- fw_match_traits2 %>% dplyr::distinct()

    return(tibble::as_tibble(fw_match_traits3))
  } else {
    fw_match_traits3 <- fw_match_traits2 %>%
      dplyr::filter(pres_sum >= certainty_val) %>%
      dplyr::distinct()
    if (!hide_printout) print(paste(length(setdiff(unique(fd$taxon), unique(c(fw_match_traits3$taxon_resource, fw_match_traits3$taxon_consumer)))), "taxa dropped from web"))

    if (print_dropped_taxa == TRUE) {
      if (!hide_printout) print(setdiff(unique(fd$taxon), unique(c(fw_match_traits3$taxon_resource, fw_match_traits3$taxon_consumer))))
    } else {
    }

    return(tibble::as_tibble(fw_match_traits3[, c("taxon_resource", "taxon_consumer")]))
  }
}

test_that("infer_edgelist runs without error", {

  expect_no_error(
    infer_edgelist(
      data = traits,
      cat_combo_list = feeding_rules,
      col_taxon = "species",
      hide_printout = TRUE,
      allow_self = FALSE
    )
  )

})

test_that("infer_edgelist returns expected interactions", {

  result <- infer_edgelist(
    data = traits,
    cat_combo_list = feeding_rules,
    col_taxon = "species",
    hide_printout = TRUE
  )

  result <- dplyr::arrange(result, taxon_consumer, taxon_resource)
  expected <- dplyr::arrange(edgelist, taxon_consumer, taxon_resource)

  expect_equal(result, expected)

})

test_that("certainty_req filters interactions correctly", {

  strict <- infer_edgelist(
    data = traits,
    cat_combo_list = feeding_rules,
    col_taxon = "species",
    certainty_req = "all",
    hide_printout = TRUE
  )

  relaxed <- infer_edgelist(
    data = traits,
    cat_combo_list = feeding_rules,
    col_taxon = "species",
    certainty_req = 1,
    hide_printout = TRUE
  )

  expect_true(nrow(strict) <= nrow(relaxed))

})

test_that("return_full_matrix returns expanded matrix", {

  full <- infer_edgelist(
    data = traits,
    cat_combo_list = feeding_rules,
    col_taxon = "species",
    return_full_matrix = TRUE,
    hide_printout = TRUE
  )

  expect_true("pres_sum" %in% colnames(full))
  expect_true(nrow(full) > 0)

})

test_that("missing taxon column throws error", {

  bad_data <- traits
  names(bad_data)[1] <- "wrong"

  expect_error(
    infer_edgelist(
      data = bad_data,
      cat_combo_list = feeding_rules,
      col_taxon = "species"
    ),
    "not found"
  )

})

test_that("invalid certainty_req errors", {

  expect_error(
    infer_edgelist(
      data = traits,
      cat_combo_list = feeding_rules,
      certainty_req = "invalid"
    )
  )

})

test_that("numeric size rule requires function", {

  expect_error(
    infer_edgelist(
      data = traits,
      cat_combo_list = feeding_rules,
      col_num_size = "size"
    )
  )

})

test_that("infer_edgelist correctly applies numeric size rules", {

  # 1. Create a simplified numeric dataset
  numeric_traits <- data.frame(
    species = c("Lion", "Zebra", "Grass"),
    diet = c("carnivore", "herbivore", "producer"),
    body_mass = c(190, 300, 1) # Note: Zebra is heavier than Lion
  )

  # 2. Create a basic categorical rule (Carnivores eat Herbivores)
  simple_rules <- data.frame(
    trait_type_resource = "diet",
    trait_resource = "herbivore",
    trait_type_consumer = "diet",
    trait_consumer = "carnivore"
  )

  # 3. Define a numeric rule: Consumer must be at least 50% of resource mass
  # Using your required syntax: function(res_size, con_size)
  mass_rule <- function(res_size, con_size) {
    ifelse(con_size >= (res_size * 0.5), 1, 0)
  }

  # 4. Run the function
  result <- infer_edgelist(
    data = numeric_traits,
    cat_combo_list = simple_rules,
    col_taxon = "species",
    col_num_size = "body_mass",
    num_size_rule = mass_rule,
    certainty_req = "all", # Must satisfy diet rule AND mass rule
    hide_printout = TRUE
  )

  # 5. Assertions
  # In this scenario:
  # Lion (190) eating Zebra (300) is feasible (190 >= 150)
  # If we changed the rule to con_size > res_size, it would return 0 rows.

  expect_s3_class(result, "tbl_df")
  expect_true(any(result$taxon_consumer == "Lion" & result$taxon_resource == "Zebra"))

  # Test that it fails if the rule is not met
  strict_rule <- function(res_size, con_size) {
    ifelse(con_size > res_size, 1, 0)
  }

  result_strict <- infer_edgelist(
    data = numeric_traits,
    cat_combo_list = simple_rules,
    col_taxon = "species",
    col_num_size = "body_mass",
    num_size_rule = strict_rule,
    certainty_req = "all",
    hide_printout = TRUE
  )

  # Lion is NOT larger than Zebra, so no links should be found
  expect_equal(nrow(result_strict), 0)
})

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom dplyr any_vars
#' @importFrom dplyr bind_rows
#' @importFrom dplyr filter_all
#' @importFrom dplyr sample_n
#' @importFrom stats na.omit
#' @importFrom stats setNames
#' @importFrom tidyr %>%
#' @importFrom tidyselect all_of
## usethis namespace: end
NULL

#' Example species traits data to infer feeding rules
#'
#' A mock trait dataset using four trait classes to determine interactions
#' as specified in `feeding_rules`
#'
#' @format ## `traits`
#' A data frame with 7,240 rows and 60 columns:
#' \describe{
#'   \item{species}{species name}
#'   \item{motility}{motility class of species}
#'   \item{habitat}{habitat species found in}
#'   \item{feeding}{trophic level of species}
#'   \item{size}{categorical size classes}
#'   ...
#' }
#' @source NA
"traits"

#' Example feeding rules for `traits`
#'
#' A mock trait dataset using four trait classes specified in `traits`
#' to infer feeding interactions
#'
#' @format ## `feeding_rules`
#' A data frame with 7,240 rows and 60 columns:
#' \describe{
#'   \item{trait_type_resource}{broader resource trait class - i.e. column name in traits}
#'   \item{trait_resource}{specific resource trait class - i.e. row entry in trait column}
#'   \item{trait_type_consumer}{broader consumer trait class - i.e. column name in traits}
#'   \item{trait_consumer}{specific consumer trait class - i.e. row entry in trait column}
#'   ...
#' }
#' @source NA
"feeding_rules"

#' Example feeding rules for `traits`
#'
#' A mock trait dataset using four trait classes specified in `traits`
#' to infer feeding interactions
#'
#' @format ## `feeding_rules`
#' A data frame with 7,240 rows and 60 columns:
#' \describe{
#'   \item{trait_type_resource}{broader resource trait class - i.e. column name in traits}
#'   \item{trait_resource}{specific resource trait class - i.e. row entry in trait column}
#'   \item{trait_type_consumer}{broader consumer trait class - i.e. column name in traits}
#'   \item{trait_consumer}{specific consumer trait class - i.e. row entry in trait column}
#'   ...
#' }
#' @source NA
"feeding_rules"

---
title: "Using Continuous Variables"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using Continuous Variables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Introduction

In many food webs, a consumer's ability to eat a resource is limited by their relative body sizes. The `infer_edgelist()` function allows you to layer a numerical rule (like a predator-prey mass ratio) on top of your categorical trait matching.

# Prepare Numerical Data

First, ensure your taxon data includes a numeric column for size (e.g., body mass in kg or length in cm). Here we will modify our traits dataframe

```{r, modify_traits}

library(pfwim)
library(dplyr)

# Adding body mass to our traits
traits_numeric <- traits %>%
  mutate(body_mass = case_when(
    species == "polar_bear" ~ 450,
    species == "seal"       ~ 100,
    species == "orca"       ~ 3000,
    species == "cod"        ~ 5,
    species == "plankton"   ~ 0.0001,
    TRUE                    ~ 10 # Default for others
  ))

head(traits_numeric)

```

# Define the Size Rule Function

The num_size_rule argument requires a function that takes two inputs (res_size, con_size) and returns 1 (feasible) or 0 (not feasible).

```{r, size_rule}

my_size_rule <- function(res_size, con_size) {
  ratio <- con_size / res_size
  ifelse(ratio >= 2 & ratio <= 100, 1, 0)
}

```

# Run Inference with Size Constraints

When `col_num_size` is provided, the function treats it as an additional trait type. If `certainty_req = "all"`, the interaction must satisfy all categorical matches AND the numerical size rule.

```{r, build_list}

edgelist_size <- infer_edgelist(
  data = traits_numeric,
  cat_combo_list = feeding_rules,
  col_taxon = "species",
  col_num_size = "body_mass",    # Point to the numeric column
  num_size_rule = my_size_rule,   # Apply our custom logic
  certainty_req = "all",
  hide_printout = TRUE
)

head(edgelist_size)
```

---
title: "Building a Metaweb"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Building a Metaweb}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Introduction

A metaweb represents the set of all potential consumer–resource
interactions in a system. In the context of paleoecology, the metaweb
captures interactions that are trait-compatible, even if they may
not all occur simultaneously in a realised community.

The PFWIM workflow allows users to infer these potential interactions
from species trait data and categorical feeding rules.

This vignette demonstrates how to:
* Infer a metaweb edgelist using infer_edgelist()
* Convert the edgelist into an igraph network
* Visualise the resulting food web

```{r setup, message=FALSE, warning=FALSE}

library(pfwim)
library(dplyr)
library(igraph)

data("traits", package = "pfwim")
data("feeding_rules", package = "pfwim")

```

# Inferring the Metaweb

The function `infer_edgelist()` evaluates trait compatibility between
species according to a set of categorical feeding rules. It returns an
edgelist, where each row represents a feasible interaction.

```{r}


metaweb_el <- infer_edgelist(
  data = traits,
  cat_combo_list = feeding_rules,
  col_taxon = "species",
  certainty_req = "all",
  hide_printout = TRUE
)

head(metaweb_el)

```

Typical edgelists contain:

* *consumer* – the feeding species
* *resource* – the prey species

These edges define the structure of the potential food web.

# Converting the Edgelist to an igraph Network

The edgelist can be directly converted into an igraph object, which
allows users to compute network statistics and generate visualisations.

``` {r}


metaweb_graph <- graph_from_data_frame(
  metaweb_el,
  directed = TRUE
)

metaweb_graph

```

# Visualising the Metaweb

Food webs are directed networks where edges flow from resource → consumer.

```{r}


set.seed(66)

plot(
  metaweb_graph,
  vertex.size = 35,
  vertex.label.cex = 0.6,
  edge.arrow.size = 0.3,
  layout = layout_with_fr(metaweb_graph),
  main = "Metaweb"
)

```

This visualisation represents the potential trophic structure of the
community based purely on trait compatibility.

Because all feasible interactions are included, metawebs are typically
much denser than observed ecological networks.

---
title: "PFWIM Workflow"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{PFWIM Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Introduction

This vignette demonstrates how to use the PFWIM package to infer
consumer–resource interactions from trait data, and generate
hypothetical realized food webs using a power-law link distribution.

This workflow demonstrates:

1. Inferring feasible interactions with `infer_edgelist()`.
2. Downsampling interactions into hypothetical realized webs with `powerlaw_prey()`.
3. Customizing link distributions via the `func` argument.

Users can now use these realised webs for **network analyses**,  
simulation studies, or **comparisons to fossil and modern food webs**.

```{r setup, message=FALSE, warning=FALSE}
library(dplyr)
library(tidyr)
library(pfwim)

# Load example data included in the package
data("traits", package = "pfwim")
data("feeding_rules", package = "pfwim")
```

## Data Structure Requirements

Before running the inference model, ensure your data is formatted correctly. The `infer_edgelist()` function requires two specific data.frame objects.

### Taxon Trait Data (`data`)

This table contains the taxa along with their physical or ecological characteristics. Each row is a unique taxon, and columns represent specific traits.

Taxon Column: A unique identifier for each species or group (*e.g.,* "species").

Trait Columns: Categorical traits (*e.g.,* habitat, tiering, motility) or numerical traits (*e.g.,* body size).

```{r trait-preview}
# Preview the input structure
head(traits)
```

### Feeding Rules (`cat_combo_list`)

This table acts as the logic engine. It defines which consumer traits are compatible with which resource traits. It needs to contain both the broader trait category (column names in the trait data frame) as well as the specific trait class (the row entries for each trait column)

```{r rules-preview}
# Preview the input structure
head(feeding_rules)
```

## Infer Edgelist

Infer interactions using categorical trait rules

```{r infer-edgelist}

edgelist <- infer_edgelist(
  data = traits,
  cat_combo_list = feeding_rules,
  col_taxon = "species",
  certainty_req = "all",
  hide_printout = TRUE
)

head(edgelist)
```

## Generate Realised Webs with Power-Law

Generate 5 hypothetical realised webs

```{r powerlaw-webs}
realised_webs <- powerlaw_prey(
  el = edgelist,
  n_samp = 5,
  y = 2.5
)

# Inspect first realised web
realised_webs[[1]]
```

## Customizing the Power-Law Distribution

```{r custom-func}
# Define a custom in-degree distribution function
custom_func <- function(r, M, y) (M - r + 1)^(-y)

realised_webs_custom <- powerlaw_prey(
  el = edgelist,
  n_samp = 5,
  y = 2,
  func = custom_func
)

# First custom web
realised_webs_custom[[1]]
```

---
title: "Generating Realised Food Webs"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Generating Realised Food Webs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Introduction

The metaweb contains all trait-compatible interactions, but real
ecological networks typically contain far fewer links.
To simulate realistic food webs, PFWIM provides the function
`powerlaw_prey()`, which downsamples interactions from the metaweb
according to a power-law prey degree distribution.

This vignette demonstrates how to:

* Generate a metaweb edgelist
* Downsample interactions to create realised webs
* Compare the metaweb and realised networks using igraph

```{r, message=FALSE, warning=FALSE}


library(pfwim)
library(dplyr)
library(igraph)

data("traits", package = "pfwim")
data("feeding_rules", package = "pfwim")

```

# Generate the Metaweb

```{r, message=FALSE, warning=FALSE}


metaweb_el <- infer_edgelist(
  data = traits,
  cat_combo_list = feeding_rules,
  col_taxon = "species",
  certainty_req = "all",
  hide_printout = TRUE
)

```

# Generating Multiple Realised Webs

The function `powerlaw_prey()` randomly samples prey for each consumer
according to a power-law distribution, producing networks that more
closely resemble empirical food webs.

Because realised webs are generated via stochastic downsampling, it is
often useful to generate multiple network realisations. This allows
users to explore variability in network structure and perform
simulation-based analyses.

The argument `n_samp` controls how many realised webs are generated.

```{r, message=FALSE, warning=FALSE}

realised_webs <- powerlaw_prey(
  el = metaweb_el,
  n_samp = 5,
  y = 2.5
)

```

The result is a list of edgelists, where each element represents one
simulated realised food web.

```{r, message=FALSE, warning=FALSE}

length(realised_webs)

```

Each entry in the list can be accessed individually.

```{r}

realised_webs[[1]]
realised_webs[[2]]

```

# Creating a consensus Realised Web

When generating multiple realised food webs, it can be useful to
construct a single representative network that summarises the
interactions observed across all simulations.

This can be done by counting how frequently each interaction occurs
across the simulated webs.

First, combine all realised edgelists into one table. Here the `.id` 
column identifies which realised web each interaction came from.

```{r}

library(dplyr)

combined_edges <- dplyr::bind_rows(
  lapply(realised_webs, as.data.frame),
  .id = "web_id"
)

head(combined_edges)

```

Next, count how often each interaction appears across the simulations.
The column `freq` indicates the number of realised webs in which each
interaction occurs.

```{r}

edge_frequency <- combined_edges %>%
  count(resource, consumer, name = "freq")

edge_frequency

```

A simple consensus rule is to keep interactions that appear in at least
half of the simulations. This produces a consensus realised web.

```{r}

consensus_el <- edge_frequency %>%
  filter(freq >= 3)

```

# Visualising the Consensus Web

```{r}

library(igraph)

consensus_graph <- graph_from_data_frame(
  consensus_el,
  directed = TRUE
)

plot(
  consensus_graph,
  vertex.size = 35,
  vertex.label.cex = 0.7,
  edge.arrow.size = 0.3,
  main = "Consensus Realised Web"
)

```

# Weighted Consensus Web

Instead of filtering interactions, one can construct a weighted
network where edge weights represent interaction frequency. In 
this case, the `freq` column becomes the edge weight, representing
how consistently each interaction occurs across simulations.

```{r}

weighted_graph <- graph_from_data_frame(
  edge_frequency,
  directed = TRUE
)

plot(
  weighted_graph,
  vertex.size = 35,
  vertex.label.cex = 0.7,
  edge.width = E(weighted_graph)$freq,
  edge.arrow.size = 0.3,
  main = "Weighted Realised Web"
)

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downsample.R
\name{sample_pdf}
\alias{sample_pdf}
\title{Sample integers according to a discrete probability distribution}
\usage{
sample_pdf(
  M = 100,
  y = 2.5,
  func = function(r, M, y) exp(-r/(exp((y - 1) * (log(M)/y)))),
  n_samp = 100
)
}
\arguments{
\item{M}{Integer. Maximum value to sample (upper bound of 1:M).}

\item{y}{Numeric. Parameter controlling the distribution shape.}

\item{func}{Function. Probability function of r (1:M), M, and y.
Default is the PFWIM exponential–power law function:
\code{function(r, M, y) exp(-r / (exp((y - 1) * (log(M) / y))))}.}

\item{n_samp}{Integer. Number of random draws.}
}
\value{
Integer vector of length \code{n_samp} with values in 1:M.
}
\description{
Generates random draws from 1:M with probabilities defined by a
user-specified function (default is the PFWIM mixed exponential–power law
distribution).
}
\examples{
# Generate 3 replicate webs from a small edgelist
edgelist <- data.frame(
  res_node_node_name_inferred = c("plankton","plant_1","plant_2"),
  con_node_node_name_inferred = c("cod","rat","deer")
)
webs <- powerlaw_prey(edgelist, n_samp = 3, y = 2.5)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downsample.R
\name{powerlaw_prey}
\alias{powerlaw_prey}
\title{Generate hypothetical realised webs using a power-law link distribution}
\usage{
powerlaw_prey(
  el,
  n_samp = 50,
  y = 2.5,
  func = function(r, M, y) exp(-r/(exp((y - 1) * (log(M)/y))))
)
}
\arguments{
\item{el}{Data frame or matrix containing a feasible consumer–resource
edgelist. Column 1 = resource, Column 2 = consumer.}

\item{n_samp}{Integer. Number of replicate realised webs to generate. Default = 50.}

\item{y}{Numeric. Parameter controlling the shape of the power-law distribution.
Default = 2.5.}

\item{func}{Function. Probability function of the in-degree \code{r}, total prey
richness \code{M}, and parameter \code{y}. Must return a numeric value > 0.
Default:
\code{function(r, M, y) exp(-r / (exp((y - 1) * (log(M) / y))))}.}
}
\value{
A list of length \code{n_samp}. Each element is a data frame
representing a realised food web edgelist with two columns:
\describe{
\item{resource}{Resource species}
\item{consumer}{Consumer species}
}
}
\description{
PFWIM generates a series of replicate hypothetical realised food webs by
reducing the feasible links for each consumer to match a target link
distribution. The default distribution is a mixed exponential–power law
in-degree distribution as described in Shaw (2024) and Roopnarine (2006).
}
\details{
For each consumer in \code{el}, the number of prey links in a realised
web is sampled according to the distribution defined by \code{func}. The sampled
prey are drawn randomly without exceeding the maximum feasible prey for that
consumer.
}
\examples{
# Infer a minimal edgelist
edgelist <- infer_edgelist(
  data = data.frame(
    species = c("plankton","plant_1","plant_2","cod","rat","deer"),
    feeding = c("primary","primary","primary","secondary","secondary","secondary")
  ),
  cat_combo_list = data.frame(
    trait_type_resource = c("feeding","feeding","feeding"),
    trait_resource = c("primary","primary","primary"),
    trait_type_consumer = c("feeding","feeding","feeding"),
    trait_consumer = c("secondary","secondary","secondary")
  ),
  col_taxon = "species",
  certainty_req = "all"
)

# Generate realized webs
webs <- powerlaw_prey(edgelist, n_samp = 3, y = 2.5)

}
\references{
Shaw, J. (2024). PFWIM: Paleo Food-web Inference Model. \emph{Preprint}.
Roopnarine, P. (2006). \emph{Palaeoecology and food-web structure in fossil communities}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/infer_edgelist.R
\name{infer_edgelist}
\alias{infer_edgelist}
\title{Infer trophic interactions using PFWIM trait rules}
\usage{
infer_edgelist(
  data,
  cat_combo_list,
  col_taxon = "taxon",
  col_num_size = NULL,
  cat_trait_types = NULL,
  num_size_rule = NULL,
  certainty_req = "all",
  allow_self = TRUE,
  return_full_matrix = FALSE,
  print_dropped_taxa = FALSE,
  hide_printout = FALSE,
  ...
)
}
\arguments{
\item{data}{A \code{data.frame} containing taxa and associated trait values.
Each row represents a taxon and each column represents a trait.}

\item{cat_combo_list}{A \code{data.frame} defining allowed consumer–resource
trait combinations. Must contain columns:
\describe{
\item{trait_type_resource}{Resource trait category}
\item{trait_resource}{Resource trait value}
\item{trait_type_consumer}{Consumer trait category}
\item{trait_consumer}{Consumer trait value}
}}

\item{col_taxon}{Character string indicating the column containing taxon
names in \code{data}. Default \code{"taxon"}.}

\item{col_num_size}{Optional column name containing numerical size values
for taxa. Used when applying a numeric predator–prey size rule.}

\item{cat_trait_types}{Optional character vector specifying a subset of
categorical trait columns to use. If \code{NULL}, all traits appearing in
\code{cat_combo_list} are used.}

\item{num_size_rule}{Optional function defining the predator–prey size
feasibility rule. The function must accept two numeric vectors:
\verb{(resource_size, consumer_size)} and return \code{1} for feasible interactions
and \code{0} otherwise.

Example:

\code{function(res_size, con_size) { ifelse(res_size <= con_size, 1, 0) }}}

\item{certainty_req}{Defines how many trait rules must be satisfied for an
interaction to be considered feasible.
\describe{
\item{"all"}{All trait types must match}
\item{numeric}{Minimum number of matching trait types required}
}}

\item{allow_self}{Logical. If TRUE, allows interactions where the resource
and consumer are the same taxon (self-loops). Default is FALSE.}

\item{return_full_matrix}{Logical. If \code{TRUE}, returns the full matrix of
taxon pairs and the number of matching trait rules. If \code{FALSE}, returns
only inferred interactions as an edgelist.}

\item{print_dropped_taxa}{Logical. If \code{TRUE}, prints taxa that were removed
from the inferred food web because they have no feasible interactions.}

\item{hide_printout}{Logical. If \code{TRUE}, suppresses progress messages.}

\item{...}{Additional arguments reserved for future extensions.}
}
\value{
If \code{return_full_matrix = FALSE}:

A two-column matrix containing:
\describe{
\item{taxon_resource}{Resource taxon}
\item{taxon_consumer}{Consumer taxon}
}

If \code{return_full_matrix = TRUE}:

A matrix containing all taxon pairs and the number of trait rules satisfied.
}
\description{
Infers a consumer–resource edgelist using categorical trait matching rules
based on the PFWIM (Paleo Food Web Inference Model) described in
Shaw (2024). Interactions are inferred by comparing resource and consumer
trait combinations against a set of allowed trait rules.
}
\details{
Trait matching is performed across all trait types present in
\code{cat_combo_list}. For each potential taxon pair, the number of satisfied
trait rules is calculated. An interaction is inferred when the number of
satisfied rules meets the required threshold defined by \code{certainty_req}.

Optionally, a numerical predator–prey size rule can also be applied using
\code{num_size_rule}.

The function evaluates all possible consumer–resource taxon pairs and
determines interaction feasibility using categorical trait rules and,
optionally, a numerical size constraint. The final edgelist includes only
interactions meeting the certainty requirement.
}
\examples{
infer_edgelist(
  data = traits,
  cat_combo_list = feeding_rules,
  col_taxon = "species",
  certainty_req = "all"
)

}
\references{
Shaw, J. (2024). PFWIM: Paleo Food web Inference Model. \emph{Preprint}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{feeding_rules}
\alias{feeding_rules}
\title{Example feeding rules for \code{traits}}
\format{
\subsection{\code{feeding_rules}}{

A data frame with 7,240 rows and 60 columns:
\describe{
\item{trait_type_resource}{broader resource trait class - i.e. column name in traits}
\item{trait_resource}{specific resource trait class - i.e. row entry in trait column}
\item{trait_type_consumer}{broader consumer trait class - i.e. column name in traits}
\item{trait_consumer}{specific consumer trait class - i.e. row entry in trait column}
...
}
}

\subsection{\code{feeding_rules}}{

A data frame with 7,240 rows and 60 columns:
\describe{
\item{trait_type_resource}{broader resource trait class - i.e. column name in traits}
\item{trait_resource}{specific resource trait class - i.e. row entry in trait column}
\item{trait_type_consumer}{broader consumer trait class - i.e. column name in traits}
\item{trait_consumer}{specific consumer trait class - i.e. row entry in trait column}
...
}
}
}
\source{
NA

NA
}
\usage{
feeding_rules

feeding_rules
}
\description{
A mock trait dataset using four trait classes specified in \code{traits}
to infer feeding interactions

A mock trait dataset using four trait classes specified in \code{traits}
to infer feeding interactions
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pfim.R-package.R
\docType{package}
\name{pfwim-package}
\alias{pfwim}
\alias{pfwim-package}
\title{pfwim: Food Web Construction for Paleo Communities}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

R tools for reconstructing paleo food webs from species traits and size rules.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://beckslab.github.io/pfwim/}
  \item \url{https://github.com/BecksLab/pfwim}
}

}
\author{
\strong{Maintainer}: Tanya Strydom \email{t.strydom@sheffield.ac.uk} (\href{https://orcid.org/0000-0001-6067-1349}{ORCID})

Other contributors:
\itemize{
  \item Jack O. Shaw (\href{https://orcid.org/0000-0002-2725-2334}{ORCID}) [contributor]
  \item Alexander M. Dunhill (\href{https://orcid.org/0000-0002-8680-9163}{ORCID}) [contributor]
  \item Andrew Beckerman (\href{https://orcid.org/0000-0002-7859-8394}{ORCID}) [contributor]
  \item Baran Karapunar (\href{https://orcid.org/0000-0001-9711-1492}{ORCID}) [contributor]
  \item Tanya Strydom (\href{https://orcid.org/0000-0001-6067-1349}{ORCID}) [contributor]
  \item Jennifer A. Dunne (\href{https://orcid.org/0000-0002-3789-9561}{ORCID}) [contributor]
  \item Pincelli M. Hull (\href{https://orcid.org/0000-0001-8607-4817}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{traits}
\alias{traits}
\title{Example species traits data to infer feeding rules}
\format{
\subsection{\code{traits}}{

A data frame with 7,240 rows and 60 columns:
\describe{
\item{species}{species name}
\item{motility}{motility class of species}
\item{habitat}{habitat species found in}
\item{feeding}{trophic level of species}
\item{size}{categorical size classes}
...
}
}
}
\source{
NA
}
\usage{
traits
}
\description{
A mock trait dataset using four trait classes to determine interactions
as specified in \code{feeding_rules}
}
\keyword{datasets}
